'---------------------------------------------------------------------
'  Copyright (C) Microsoft Corporation.  All rights reserved.
' 
'THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
'KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'PARTICULAR PURPOSE.
'---------------------------------------------------------------------

Option Explicit On
Option Strict On

Imports System.Collections
Imports System.Collections.Generic
Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.ComponentModel.Design
Imports System.Diagnostics
Imports System.Drawing
Imports System.Drawing.Design
Imports System.IO
Imports System.Reflection
Imports System.Windows.Forms
Imports System.Windows.Forms.Design
Imports System.Windows.Forms.Design.Behavior

Imports DesignSurfaceExtenderLibrary.Extensions

Friend Class DesignSurfaceExtenderDesigner
    Inherits ComponentDesigner

    ''' <summary>
    ''' Backs the <see cref="ActionLists"/> property.
    ''' </summary>
    Private actionListsCollection As DesignerActionListCollection

    ''' <summary>
    ''' Backs the <see cref="Extensions"/> property.
    ''' </summary>
    Private extensionsCollection As ReadOnlyCollection(Of DesignSurfaceExtension)

    ''' <summary>
    ''' Path to the Plugins directory.
    ''' </summary>
    ''' <remarks>Change this path to the correct location for your installation.
    ''' Yes, this should go into configuration.
    ''' </remarks>
    Private Shared pluginPath As String = "C:\Projects\Designer\DesignSurfaceExtenderLibrary\DesignSurfaceExtenderLibrary\Plugins"

    ''' <summary>
    ''' Gets the available design surface extensions.
    ''' </summary>
    Friend ReadOnly Property Extensions() As ReadOnlyCollection(Of DesignSurfaceExtension)
        Get
            If Me.extensionsCollection Is Nothing Then
                Dim _extensions As List(Of DesignSurfaceExtension) = Me.FindAvailableExtensions()
                Me.extensionsCollection = New ReadOnlyCollection(Of DesignSurfaceExtension)(_extensions)
            End If
            Return Me.extensionsCollection
        End Get
    End Property

    Private Function FindAvailableExtensions() As List(Of DesignSurfaceExtension)
        Dim a As Assembly = Assembly.GetExecutingAssembly()

        Dim extensions As List(Of DesignSurfaceExtension) = Me.FindExtensionsInAssembly(a)

        Dim files() As String = Directory.GetFiles(pluginPath)

        For i As Integer = 0 To files.Length - 1
            Dim fi As FileInfo = New FileInfo(files(i))
            If fi.Extension.ToLower() = ".dll" Then
                a = Assembly.LoadFile(fi.FullName)
                extensions.AddRange(Me.FindExtensionsInAssembly(a))
            End If
        Next
        Return extensions

    End Function

    ''' <summary>
    ''' Searches the executing assembly for types which derive 
    ''' from <see cref="DesignSurfaceExtension"/>.
    ''' </summary>
    ''' <returns>A collection of objects which derive from 
    ''' <see cref="DesignSurfaceExtension"/>.</returns>
    ''' <remarks>To achieve a truly pluggable design, this method
    ''' would use configuration to check other assemblies for types 
    ''' which derive from <see cref="DesignSurfaceExtension"/>.
    ''' </remarks>
    Private Function FindExtensionsInAssembly(ByVal a As Assembly) As List(Of DesignSurfaceExtension)
        Dim extensions As List(Of DesignSurfaceExtension) = New List(Of DesignSurfaceExtension)

        Dim types() As Type = a.GetTypes()

        For i As Integer = 0 To types.Length - 1
            If types(i).IsSubclassOf(GetType(DesignSurfaceExtension)) Then
                Dim types2(0) As Type
                types2(0) = GetType(IServiceProvider)
                Dim ci As ConstructorInfo = types(i).GetConstructor( _
                    BindingFlags.NonPublic Or BindingFlags.Public Or BindingFlags.Instance, _
                    Nothing, _
                    types2, _
                    Nothing)
                If ci IsNot Nothing Then
                    Dim parameters(0) As Object
                    parameters(0) = Me.Component.Site
                    Dim extension As DesignSurfaceExtension = CType(ci.Invoke( _
                        parameters), DesignSurfaceExtension)
                    extensions.Add(extension)
                End If
            End If
        Next
        Return extensions
    End Function

    Public Overrides ReadOnly Property ActionLists() As DesignerActionListCollection
        Get
            If actionListsCollection Is Nothing Then
                actionListsCollection = New DesignerActionListCollection
                For i As Integer = 0 To Me.Extensions.Count - 1
                    actionListsCollection.Add(Me.Extensions(i).ActionList)
                Next
                actionListsCollection.Add(DesignSurfaceExtenderActionList.create( _
                    Me.Component, Me.extensionsCollection))
            End If
            Return actionListsCollection
        End Get
    End Property

End Class
